""" module to methods to main  """
import sys
import logging


from simulator.tools import tools_parser

logger = logging.getLogger(__name__)


def main():
    try:
        sys.exit(tools_parser(sys.argv[1:]))
    except KeyboardInterrupt:
        # É convencionado no shell que o programa finalizado pelo signal de
        # código N deve retornar o código N + 128.
        sys.exit(130)
    except Exception as exc:
        logger.exception(
            "erro durante a execução da função " "'tools_parser' com os args %s",
            sys.argv[1:],
        )
        sys.exit("Um erro inexperado ocorreu: %s" % exc)


if __name__ == "__main__":
    sys.exit(tools_parser(sys.argv[1:]))



"""
Geração dos DataSets de Clientes, Produtos
"""
import random
from faker import Faker
from simulator.generate.utils import custom_uuid, random_sku

import pandas as pd
import numpy as np


fake = Faker("pt-BR")


def customers(number_of_rows: int) -> pd.DataFrame:
    ### Gerando DataSet de Clientes

    columns_customers = ["uuid", "name"]
    df_customers = pd.DataFrame(
        columns=columns_customers,
        data=[[custom_uuid(_id), fake.name()] for _id in np.arange(0, number_of_rows)],
    )
    return df_customers


def products(number_of_rows: int) -> pd.DataFrame:
    ### Gerando DataSet de Produtos

    columns_products = ["sku", "title", "description", "price"]
    df_products = pd.DataFrame(
        columns=columns_products,
        data=[
            [random_sku(_id), fake.sentence(), fake.text(), random.randint(1, 1000)]
            for _id in np.arange(0, number_of_rows)
        ],
    )
    return df_products



import requests
import random
import time


class SendData:
    def __init__(self, hostname, ga_config):

        self._hostname = hostname
        self._ga_config = ga_config

    def _ga_cid(self):
        prefix = random.randint(1, 0x7FFFFFFF)
        epoch = time.time()
        return f"{prefix}.{epoch}"

    def _ga_send_data(self, data):
        # Send Data to Google Analytics
        # https://developers.google.com/analytics/devguides/collection/protocol/v1/devguide#event

        payload = {
            "payload_data": "",
        }
        payload.update(data)

        result = requests.get(
            "https://ssl.google-analytics.com/collect", params=payload
        )

        result.raise_for_status()
        return result

    def ga_send_pageview(self, page, title, product_id, user_id=None):
        # Send Pageview Function for Server-Side Google Analytics
        data = dict(
            v=1,
            tid=self._ga_config,
            cid=self._ga_cid(),
            t="pageview",
            dh=self._hostname,  # Document Hostname "gearside.com"
            dp=page,  # Page "/something"
            dt=title,  # Title
            cd1=product_id,  # Custom dimension
        )
        if user_id:
            data["uid"] = user_id

        return self._ga_send_data(data)



""" module to utils of logger app """
import os
from logging import config as l_config


def configure_logger():

    SUB_DICT_CONFIG = {"level": "DEBUG", "handlers": [], "propagate": False}

    l_config.dictConfig(
        {
            "version": 1,
            "formatters": {
                "default": {
                    "format": "%(asctime)s %(levelname)-5.5s [%(name)s] %(message)s",
                    "datefmt": "%Y-%m-%d %H:%M:%S",
                }
            },
            "handlers": {
                "console": {
                    "level": "DEBUG",
                    "class": "logging.StreamHandler",
                    "formatter": "default",
                    "stream": "ext://sys.stdout",
                }
            },
            "loggers": {"faker.factory": SUB_DICT_CONFIG,},
            "root": {"level": "DEBUG", "handlers": ["console"]},
        }
    )



import logging
import pandas as pd
import numpy as np

from simulator.send_data_ga.client import SendData

logger = logging.getLogger(__name__)


def pageview(
    df_customers: pd.DataFrame, df_products: pd.DataFrame, interactions: int
) -> None:

    ga = SendData(
        "ecommerce-raw-bart.s3-website-us-east-1.amazonaws.com", "UA-166855025-1"
    )

    for _id in np.arange(0, interactions):
        customer = df_customers.sample().iloc[0]
        product = df_products.sample().iloc[0]

        logger.info(f'\tSend Product {product["sku"]} and Customer {customer["name"]}')

        r = ga.ga_send_pageview(
            f'/products/{product["sku"]}.html',
            f'Bart Recomendation | {product["title"]}',
            product["sku"],
            customer["uuid"],
        )
        logger.info(f"Retorno GA: {r.status_code}")



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': '.'}

packages = \
['simulator', 'simulator.generate', 'simulator.send_data_ga', 'simulator.utils']

package_data = \
{'': ['*']}

install_requires = \
['faker>=4.1.0,<5.0.0', 'pandas>=1.0.3,<2.0.0', 'requests>=2.23.0,<3.0.0']

entry_points = \
{'console_scripts': ['simulator = simulator.main:main']}

setup_kwargs = {
    'name': 'bart-simulator',
    'version': '0.1.0',
    'description': 'Send event views to Google Analytics and Generator customers or products',
    'long_description': "# Bart Simulator (CLI)\n\nSend event views to Google Analytics and Generator customers or products\n\n## Install\n```\n$ pip install bart-simulator\n```\n\n## Usage\n\n```\nusage: simulator [-h] [--loglevel LOGLEVEL] [--version]  ...\n\nSend event views to Google Analytics and Generator customers or products data-\nset (bart-recs CLI)\n\noptional arguments:\n  -h, --help           show this help message and exit\n  --loglevel LOGLEVEL\n  --version            show program's version number and exit\n\nCommands:\n\n    generation         Gera os data set simulados para as recomendações\n    send-data-ga       Envia dados simulados para o Google Analytics\n```\n\n#### Generate DataSets\n```\n# Generation customers csv\n$ simulator generation customers -f csv\n\n# Generation customers json\n$ simulator generation customers -f json\n\n# Generation products csv\n$ simulator generation products -f csv\n\n# Generation products json\n$ simulator generation products -f json\n\n```\n#### Full Options\n```\nusage: simulator generation [-h] [--desc-path DESC_PATH] [--rows ROWS]\n                            --format {csv,json} [{csv,json} ...]\n                            {customers,products}\n\npositional arguments:\n  {customers,products}  Arquivo que sera gerado pelo processo\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --desc-path DESC_PATH, -d DESC_PATH\n                        Pasta onde sera salvo os dataset gerados\n  --rows ROWS, -r ROWS  Quantidades de Linhas geradas\n  --format {csv,json} [{csv,json} ...], -f {csv,json} [{csv,json} ...]\n                        Formato do arquivo de saida que sera salvo,pode ser\n                        adiciona mais de um tipo ao mesmo tempo\n```\n\n#### Send events to GA\n\n```\n$ simulator send-data-ga pageview \\\n    -c https://github.com/cesarbruschetta/bart-recs/datasets/customers.csv \\\n    -p https://github.com/cesarbruschetta/bart-recs/datasets/products.csv \\\n    -i 10\n```\n\n#### Full Options\n```\nusage: simulator send-data-ga [-h] --customers CUSTOMERS --products PRODUCTS\n                              [--interactions INTERACTIONS]\n                              {pageview}\n\npositional arguments:\n  {pageview}            Tipo de evento que sera enviado ao GA\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --customers CUSTOMERS, -c CUSTOMERS\n                        Caminho para o dataset de customers, em csv\n  --products PRODUCTS, -p PRODUCTS\n                        Caminho para o dataset de products, em csv\n  --interactions INTERACTIONS, -i INTERACTIONS\n                        Quantidades de interações geradas\n```\n\n\n\n## Licence\n[Apache License](./LICENSE) - Version 2.0, January 2004 \n\n",
    'author': 'Cesar Augusto',
    'author_email': 'cesarabruschetta@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/cesarbruschetta/bart-recs',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from hashlib import md5
from uuid import UUID


def custom_uuid(id: int) -> str:
    return str(UUID(md5(str(id).encode("utf-8")).hexdigest()))


def random_sku(id: int) -> str:
    return md5(str(id).encode("utf-8")).hexdigest()[:8]



"""  """
import logging
import pkg_resources
import argparse
import os
import pandas as pd

from simulator.generate import datasets
from simulator.send_data_ga import data_types


base_path = os.getcwd()


def tools_parser(sargs):
    parser = argparse.ArgumentParser(
        description="Send event views to Google Analytics and Generator "
        "customers or products data-set (bart-recs CLI)"
    )
    parser.add_argument("--loglevel", default="INFO")

    packtools_version = pkg_resources.get_distribution("bart-simulator").version
    parser.add_argument("--version", action="version", version=packtools_version)

    subparsers = parser.add_subparsers(title="Commands", metavar="", dest="command")

    # GENERATION Datasets
    parents_generation = argparse.ArgumentParser(add_help=False)
    parents_generation.add_argument(
        "script",
        choices=["customers", "products"],
        help="Arquivo que sera gerado pelo processo",
    )
    parents_generation.add_argument(
        "--desc-path",
        "-d",
        default=base_path,
        help="Pasta onde sera salvo os dataset gerados",
    )
    parents_generation.add_argument(
        "--rows", "-r", default=1000, help="Quantidades de Linhas geradas", type=int,
    )
    parents_generation.add_argument(
        "--format",
        "-f",
        default="csv",
        choices=["csv", "json"],
        nargs="+",
        help="Formato do arquivo de saida que sera salvo, "
        "pode ser adiciona mais de um tipo ao mesmo tempo ",
        required=True,
    )

    subparsers.add_parser(
        "generation",
        help="Gera os data set simulados para as recomendações",
        parents=[parents_generation,],
    )

    # Send Ping GA
    parents_send_data_ga = argparse.ArgumentParser(add_help=False)
    parents_send_data_ga.add_argument(
        "event", choices=["pageview",], help="Tipo de evento que sera enviado ao GA",
    )
    parents_send_data_ga.add_argument(
        "--customers",
        "-c",
        required=True,
        help="Caminho para o dataset de customers, em csv",
    )
    parents_send_data_ga.add_argument(
        "--products",
        "-p",
        required=True,
        help="Caminho para o dataset de products, em csv",
    )
    parents_send_data_ga.add_argument(
        "--interactions",
        "-i",
        default=1000,
        help="Quantidades de interações geradas",
        type=int,
    )

    subparsers.add_parser(
        "send-data-ga",
        help="Envia dados simulados para o Google Analytics",
        parents=[parents_send_data_ga,],
    )

    ################################################################################################
    args = parser.parse_args(sargs)

    # CHANGE LOGGER
    level = getattr(logging, args.loglevel.upper())
    logger = logging.getLogger()
    logger.setLevel(level)

    if args.command == "generation":
        logger.info("Iniciando o processamento...")

        func = getattr(datasets, args.script)
        logger.info(f"Gerando arquivo {args.script}")
        dataset = func(args.rows)

        if "csv" in args.format:
            dataset.to_csv(os.path.join(args.desc_path, f"{args.script}.csv"))
            logger.info("Arquivo csv salvo com sucesso!")

        if "json" in args.format:
            dataset.to_json(
                os.path.join(args.desc_path, f"{args.script}.json"),
                orient="records",
                indent=4,
            )
            logger.info("Arquivo json salvo com sucesso!")
        logger.info(
            f"Processamento Finalizado, arquivos salvos na pasta {args.desc_path}"
        )

    elif args.command == "send-data-ga":
        logger.info("Iniciando o processamento...")

        func = getattr(data_types, args.event)
        result = func(
            df_customers=pd.read_csv(args.customers),
            df_products=pd.read_csv(args.products),
            interactions=args.interactions,
        )
        logger.info(
            f"Processamento Finalizado, Enviado {args.interactions} {args.event} para o GA"
        )

    else:
        raise SystemExit(
            "Vc deve escolher algum parametro, ou '--help' ou '-h' para ajuda"
        )

    return 0
